package com.supervision.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import com.supervision.domain.IrKnowledge;
import com.supervision.domain.IrSessionParam;
import com.supervision.domain.IrSqlParam;
import com.supervision.service.IrKnowledgeService;
import com.supervision.service.IrSessionParamService;
import com.supervision.service.IrSqlParamService;
import com.supervision.service.QueryTemplateProcessor;
import com.supervision.util.ParamTypeConverter;
import com.supervision.util.freemark.StringTemplateConfig;
import com.supervision.util.mybatis.RowSqlMapper;
import freemarker.template.TemplateException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class QueryTemplateProcessorImpl implements QueryTemplateProcessor {

    private final IrKnowledgeService knowledgeService;

    private final IrSqlParamService sqlParamService;

    private final IrSessionParamService sessionParamService;

    private final RowSqlMapper rowSqlMapper;
    @Override
    public String process(String templateId,String sessionId) throws TemplateException, IOException {

        Assert.notEmpty(templateId, "模板ID不能为空");

        // 根据模板ID查询查询sql模板和结果模板
        IrKnowledge knowledge = knowledgeService.getById(templateId);
        Assert.notNull(knowledge, "知识不存在！");
        Assert.notEmpty(knowledge.getSqlTemplate(), "模板内容不能为空");
        Assert.notEmpty(knowledge.getResultTemplate(), "结果模板不能为空");
        Assert.notNull(knowledge.getResultType(), "结果类型不能为空");

        // 查询必填的参数
        List<IrSqlParam> paramsList = sqlParamService.lambdaQuery().eq(IrSqlParam::getKnowledgeId, templateId).list();

        // 校验必填参数
        List<IrSessionParam> sessionParams = sessionParamService.lambdaQuery().eq(IrSessionParam::getSessionId, sessionId).list();
        Set<String> sessionParamsNames = sessionParams.stream().map(IrSessionParam::getParamName).collect(Collectors.toSet());
        paramsList.stream().filter(param -> Integer.valueOf(1).equals(param.getParamRequire()))
                .forEach(param -> Assert.isTrue(sessionParamsNames.contains(param.getParamName()), "参数["+param.getParamName()+"]未设置！"));

        // 执行查询sql
        List<Map<String, Object>> maps = rowSqlMapper.selectList(knowledge.getSqlTemplate(), getParams(paramsList, sessionParams));

        // 组装返回结果
        if (Integer.valueOf(1).equals(knowledge.getResultType())){
           return StringTemplateConfig.getInstance().process(knowledge.getResultTemplate(), CollUtil.getFirst(maps));
        }
        if (Integer.valueOf(2).equals(knowledge.getResultType())){
            return StringTemplateConfig.getInstance().process(knowledge.getResultTemplate(), maps);
        }
        // todo:字节类型暂不支持
        if (Integer.valueOf(3).equals(knowledge.getResultType())){
            return null;
        }
        return null;
    }

    private Map<String, Object> getParams(List<IrSqlParam> paramsList, List<IrSessionParam> sessionParams) {
        Map<String, Object> params = new HashMap<>();
        ParamTypeConverter converter = ParamTypeConverter.getInstance();
        for (IrSqlParam param : paramsList) {
            Optional<IrSessionParam> sessionParam = sessionParams.stream().filter(v -> v.getParamName().equals(param.getParamName())).findFirst();
            sessionParam.ifPresent(irSessionParam -> params.put(param.getParamName(), converter.convert(param.getParamType(), irSessionParam.getParamValue())));
        }
        return params;
    }


}
